from os.path import join
import sys
import os
import random
import shutil

try:
    import configparser
except ImportError:
    # Python 2.x fallback
    import ConfigParser as configparser
from time import time

# COMPSs/PyCOMPSs imports
from pycompss.api.task import task
from pycompss.api.binary import binary
from pycompss.api.constraint import constraint
from pycompss.api.mpi import mpi
from pycompss.api.api import compss_wait_on
from pycompss.api.api import compss_barrier
from pycompss.api.api import TaskGroup
from pycompss.api.api import compss_barrier_group
from pycompss.api.parameter import *
from pycompss.api.on_failure import on_failure

# project imports
from esm_ensemble_init import *
from esm_pruning import *


# date
# mpirun -n 288 fesom.x > fesom2_log.out
# date

# check: https://able.bio/rhett/how-to-set-and-get-environment-variables-in-python--274rgt5
# TODO: working_dir some Guys from COPSs are working on a version to take the working path from task decorator

#@mpi(binary="${FESOM_WORKINGDIR}/${FESOM_EXE}", runner="mpirun", processes="${FESOM_CORES}", working_dir="${FESOM_WORKINGDIR}", fail_by_exit_value=True)
@on_failure(management ='CANCEL_SUCCESSORS')
@mpi(binary=os.environ['FESOM_WORKINGDIR'] + "/${FESOM_EXE}", runner="mpirun", processes="${FESOM_CORES}", working_dir=os.environ['FESOM_WORKINGDIR'], fail_by_exit_value=True)
@task(log_file={Type:FILE_OUT, StdIOStream:STDOUT}, working_dir={Type:IN})
def esm_simulation(log_file, working_dir):
   pass


def esm_pruning():
   # 1 - connect to hecuba and obtain values, the datamodel generared by hecuba when loading the defined
   print("#################### checking state of member x")
   # 2 - for each simulation to be stopped, remove it from the task group

   # 3 - remove its data from hecuba and remove its working directory and generated outputs

# takes the ensemble config file and prepares folders and config files


   # 3 - prepare config files for each simulation: replace placeholders in namelists for each member of the ensemble (start date), create fesom.clock. etc

# function to run ESM ensemble, each one should have different start dates or initial conditions/ forcings
# for making sense
def esm_ensemble(exp_id):
   # 1 - launch dinamic workflow that will analize model generated data in hecuba

   # 2- Creation of group
   with TaskGroup(str(exp_id), False):
      #3 - Launch the ensemble members
      sdates_list = (exp_settings['common']['ensemble_start_dates']).split()
      # todo : iterate also on the number of chunks, it may be necesary to trick FESOM2
      # todo : iterate also on the number of chunks, it may be necesary to trick FESOM2
      for sdate in sdates_list:
         working_dir = exp_settings['common']['top_working_dir'] + "/" + sdate
         output_dir = exp_settings['common']['output_dir'] + "/" + sdate
         os.environ['FESOM_WORKINGDIR'] = working_dir
         nchunks = int(exp_settings['common']['number_chunks'])
         for chunk in range(1, nchunks):
            log = working_dir + "/" + "fesom2_" + str(exp_id) + "_" + str(sdate) + "_" + str(chunk) + ".out"
            print("################################################### launching simulation in " + working_dir + "###################################################")
            esm_simulation(log, working_dir)
            # move files to the right folder or also renaming can be an option
            shutil.move(working_dir + "/*.nc", output_dir + "/" + str(chunk) + "/")
            # todo : it may need to add a barrier so simulation and pruning method does not overlap at all
            print("################################################### checking state for member " + sdate + "###################################################")
            # esm_pruning(exp_id,sdate)
   compss_barrier_group(str(exp_id))


if __name__ == "__main__":
   print("Running FESOM2 -Pycompss")
   # 0 - create a ramdon nr for the experiment id
   exp_id = random.randint(100000, 999999)
   # prepare folder structure/configuration
   exp_settings = compss_wait_on(esm_ensemble_init(exp_id))
  # exp_settings =  compss_wait_on(exp_settings)
   print("##################################### Initialization completed ####################################")
   # run the experiment
   esm_ensemble(exp_id)


